#ifndef _HDT_RNBW_TREE_H
#define _HDT_RNBW_TREE_H

// ------------------------------------------------------------- //
// hdt_rnbw_tree.h: header file for the hdt_rnbw_trees           //
//                                                               //
// comment: hdt_rnbw_trees are a modified version of David       //
//	    Alberts' rnbw_trees (derived from rnb_trees).	 //
//	    They have a non-negative weight and a bitmap	 //
//	    at each node and subtree weights.                    //
// ------------------------------------------------------------- //

#include <rnb_tree.h>
#include <weights.h>

class hdt_rnbw_node_struct;

typedef hdt_rnbw_node_struct* hdt_rnbw_node;
typedef hdt_rnbw_node hdt_rnbw_tree;


class hdt_rnbw_node_struct : public rnb_node_struct {

public:

  hdt_rnbw_node_struct( Weight w) : rnb_node_struct() 
    { weight = sub_weight = w; }
  // construct a new tree containing just one node with weight w.
  // By default each node gets a weight of one.
  // Prec.: w >= 0

  ~hdt_rnbw_node_struct() {};

  Weight get_weight() { return weight; }
  // returns the weight of this node

  Weight get_subtree_weight() { return sub_weight; }
  // returns the weight of the subtree rooted at this node

  void set_weight(Weight w);
  // sets the weight of this node to w
  // Prec.: w >= 0

  void add_weight(Weight w);
  // adds w to the weight of this node
  // Prec.: w >= -(weight of this node)

  //  friend hdt_rnbw_node hdt_rnbw_locate(hdt_rnbw_tree t, int w, int& offset);
  // returns the node in the tree rooted at t which corresponds to
  // w with respect to In-order.
  // Prec.: 0 < w <= weight of tree rooted at t

  // --- Conversion Functions ---
  hdt_rnbw_node parent() { return (hdt_rnbw_node) par; }
  hdt_rnbw_node left_child() { return (hdt_rnbw_node) child[rnb_left]; }
  hdt_rnbw_node right_child() { return (hdt_rnbw_node) child[rnb_right]; }
  hdt_rnbw_tree find_root() {return (hdt_rnbw_tree) rnb_node_struct::find_root(); }
  hdt_rnbw_node pred() { return (hdt_rnbw_node) rnb_node_struct::pred(); }
  hdt_rnbw_node succ() { return (hdt_rnbw_node) rnb_node_struct::succ(); }
  friend inline hdt_rnbw_tree hdt_rnbw_join(hdt_rnbw_tree t1, hdt_rnbw_tree t2, 
				    hdt_rnbw_node dummy)
  { return (hdt_rnbw_tree) rnb_join(t1,t2,dummy); }

  virtual void print();
  // we redefine print in order to output also the additional fields

protected:

  void after_rot();
  // We fix the weight fields after a rotation. This is called as
  // a virtual function in the base class rnb_tree.

  void init();
  // This method is used to initialize the dummy node in join and split
  // after linking it to the tree(s). It is a virtual function in the base
  // class.

  virtual void isolate();
  // We fix the sub_weight fields of the ancestors of this node.

private:

  Weight weight;      // stores the weight of this node
  Weight sub_weight;  // stores the weight of the subtree rooted at this node
};

#endif











