// ---------------------------------------------------------- //
// fast_query.c: fast_query dynamic data structure for        //
//               connectivity                                 //
// version 1.0 Nov 2 1995, David Alberts                      //
// ---------------------------------------------------------- //

#include<LEDA/queue.h>
#include<LEDA/graph.h>

static GRAPH<int,int> *dc_Gp;  // pointer to original graph
static node_array<int> components;

static int bad_counter;
static int cmp_counter;

void my_bfs(node v, int count)
{ 
  queue<node>  Q;

  Q.append(v);
  components[v] = count;

  while (!Q.empty())
  { 
    v = Q.pop(); 
    edge e;
    forall_inout_edges(e,v) 
    {
      node u = (v == source(e)) ? target(e) : source(e);
      if(components[u] != count)
      {
        components[u] = count;
        (*dc_Gp)[e] = 2;
        Q.append(u);
      }
      else
      {
        if((*dc_Gp)[e]==2) (*dc_Gp)[e] = true;
        else               (*dc_Gp)[e] = false;
      }
    }
  }
} 

void init_components(void)
// computes connected components of dc_Gp and dc_forest
// components[v] = i  iff v in component i f.a. vertices v of *dc_Gp
// (*dc_Gp)[e] = true iff e in spanning forest f.a. edges e of *dc_Gp
{ 
  // set node and egde values in to initial values
  components.init(*dc_Gp,-1);
  edge e;
  forall_edges(e,*dc_Gp) dc_Gp->assign(e,false);

  int count = 0;
  node v;
  forall_nodes(v,*dc_Gp) 
  {
    if (components[v] == -1) 
    {
      my_bfs(v,count);
      count++; 
    }
  }
}


void fq_init(GRAPH<int,int>& initial_G)
// initializes the data structure
{
  dc_Gp=&(initial_G);

  // calculate a spanning forest and component numbers
  init_components();

  bad_counter = 0;
  cmp_counter = initial_G.number_of_nodes() + 1;
}

int fq_query(node v, node w)
// returns true iff v and w are connected in G
// precondition: v, w are nodes of G
{
  return (components[v] == components[w]);	
}

edge fq_insert(node v, node w)
// inserts an edge between v and w in G
// precondition: v, w are nodes of G and they are not already
//               connected by a single edge
{
  edge e = dc_Gp->new_edge(v,w,false);
  if(!fq_query(v,w))
  {
    (*dc_Gp)[e] = true;
    my_bfs(v,cmp_counter++);
    bad_counter++;
  }
  return e;
}

void fq_delete(edge e)
// deletes the edge e from G
// precondition: e is an edge of G
{
  if((*dc_Gp)[e])
  {
    dc_Gp->del_edge(e);
    my_bfs(source(e),cmp_counter++);
    bad_counter++;
  }
  else dc_Gp->del_edge(e);
}

int fq_bad()
{
  return bad_counter;
}
