// ---------------------------------------------------------- //
// fast_update.c: pseudo-dynamic algorithm for connectivity   //
//             works by using LEDA's BFS algorithm for each   //
//             query                                          //
//                                                            //
// version 1.1 Dec 6 1995, David Alberts                      //
//         removed LEDA BFS from query                        //
// version 1.0 Nov 2 1995, David Alberts                      //
// ---------------------------------------------------------- //

#include<LEDA/basic.h>
#include<LEDA/graph.h>
#include<LEDA/queue.h>

static graph* dc_Gp;

void fu_init(graph& initial_G)
// initializes the data structure
{
  dc_Gp=&(initial_G);
  initial_G.make_undirected();
}

int fu_query(node v, node w)
// returns true iff v and w are connected in G
// precondition: v, w are nodes of G
{
  if(v == w) return true;

  node u,x;
  node_array<int> visited(*dc_Gp,false);
  queue<node> Q;

  Q.append(v);
  visited[v] = true;

  while(!Q.empty())
  {
    u = Q.pop();
    forall_adj_nodes(x,u)
    {
      if(x == w) return true;
      if(!visited[x])
      {
        visited[x] = true;
        Q.append(x);
      }
    }
  }

  return false;
}

edge fu_insert(node v, node w)
// inserts an edge between v and w in G
// precondition: v, w are nodes of G and they are not already
//               connected by a single edge
{
  return dc_Gp->new_edge(v,w);
}

void fu_delete(edge e)
// deletes the edge e from G
// precondition: e is an edge of G
{
  dc_Gp->del_edge(e);
}
