// ---------------------------------------------------- //
// gen_2cliques1edge.c: Generate two cliques connected  //
//                      by one edge.                    //
//                                                      //
// usage: gen2cliques1edge #nodes/clique #ins #del #que name //
//             name = name of output files (.gr,.upd)	//
//                                                      //
// Written by H. S. Rahul				//
// ---------------------------------------------------- //

// This code is hardcoded version of gen_two-level.c

#include<ed_tree.h>
#include<sys/time.h>
#include<LEDA/graph.h>
#include<LEDA/graph_alg.h>
#include<fstream.h>
#include<math.h>
#include <stdlib.h>
#include <stdio.h>

#define MAX_NODES_PER_CITY 500
#define MAX_CITIES 500

#define MIN(u,v) ((u < v) ? u : v)
#define MAX(u,v) ((u > v) ? u : v)

int src, tgt;

graph TG;
ed_tree bridges = nil;
ed_tree del_bridges = nil;
ed_node dummy;
node nodes[MAX_CITIES*MAX_NODES_PER_CITY];

edge random_ins()
// reinsert a random deleted bridge into the current subgraph
{
  int rnd_index = int(drand48()*del_bridges->get_subtree_weight())+1;
  int offset;

  ed_node ed = ed_locate(del_bridges,rnd_index,offset);
  edge e = ed->get_corr_edge();
  ed_delete(del_bridges,ed,dummy);
  ed_insert(bridges,e,dummy);
  return e;
}

edge random_del()
// delete a random bridge from the current subgraph
{
  int rnd_index = int(drand48()*bridges->get_subtree_weight())+1;
  int offset;

  ed_node ed = ed_locate(bridges,rnd_index,offset);
  edge e = ed->get_corr_edge();
  ed_delete(bridges,ed,dummy);
  ed_insert(del_bridges,e,dummy);
  return e;
}


main(int argc, char *argv[])
{

  int num_cities, num_nodes_per_city, num_edges_per_city, num_inter_city_edges;
  int n_ins, n_del, n_que, n_ops;
  int i, j, k;

  char fname[256];
  FILE *fp;

  if (argc !=  6) {
    printf("usage: gen_2cliques1edge #nodes/clique #ins #del #que name\n");
    printf("\tgenerates two cliques connected by an edge\n");
    exit(1);
  }

  /* changed for this code */
  num_cities = 2;
  num_nodes_per_city = atoi(argv[1]);
  num_edges_per_city = num_nodes_per_city*(num_nodes_per_city-1)/2;
  num_inter_city_edges = 1;

  n_ins = atoi(argv[2]);
  n_del = atoi(argv[3]);
  n_que = atoi(argv[4]);
  n_ops = n_ins + n_del + n_que;

  sprintf(fname,"%s.gr", argv[5]);
  fp = fopen(fname, "w");

  fprintf(fp, "LEDA.GRAPH\nvoid\nvoid\n%d\n", num_cities*num_nodes_per_city);

  for(i = 0; i < num_cities*num_nodes_per_city; i++) fprintf(fp,"|{}|\n");

  fprintf(fp, "%d\n", num_cities*num_edges_per_city+num_inter_city_edges);

  for (i = 0; i < num_cities; i++) {
    for(j = 1; j < num_nodes_per_city+1; j++) {
      for (k = j+1; k < num_nodes_per_city+1; k++) {
	fprintf(fp,"%d %d 0 |{}|\n", i*num_nodes_per_city+j, i*num_nodes_per_city+k);
      }
    }
  }

  for (i = 0; i < num_cities*num_nodes_per_city; i++) nodes[i] = TG.new_node();

  dummy = new ed_node_struct(nil);

  for(j = 0; j < num_inter_city_edges; j++) {
    int u, v;
    u = int(drand48()*num_cities);
    v = u;
    while (v == u) {
      v = int(drand48()*num_cities);
    }

    /* At this point v != u */
    src = u*num_nodes_per_city + (int(drand48()*num_nodes_per_city)+1);    
    bool valid = false;
    edge e;

    while (!valid) {
      node n = nodes[src-1];
      int num_tries = 0;
      forall_adj_edges(e, n) 
	{
	  int other = index(source(e))+index(target(e))-(src-1);
	  if ((v*num_nodes_per_city <= other) && 
	      (other < ((v+1)*num_nodes_per_city))) {
	    num_tries++;
	  }
	}
      if (num_tries < num_nodes_per_city) valid = true;
      else src = u*num_nodes_per_city + (int(drand48()*num_nodes_per_city)+1);
    }


    bool found = false;

    while (!found) {
      found = true;
      tgt = v*num_nodes_per_city + (int(drand48()*num_nodes_per_city)+1);
      //cout << "src is " << src << " and tgt is " << tgt << "\n" << flush;
      node n = nodes[src-1];
      forall_adj_edges(e,n) 
	{
	  if (((index(source(e)) == src-1) && (index(target(e)) == tgt-1)) || 
	      ((index(source(e)) == tgt-1) && (index(target(e)) == src-1))) {
	    found = false;
	    break;
	  }
	}
    }
    fprintf(fp, "%d %d 0 |{}|\n", src, tgt);
    /* Insert u,v into the ed_tree bridges */
    ed_insert(bridges,TG.new_edge(nodes[src-1],nodes[tgt-1]),dummy);
  }
  
  sprintf(fname, "%s.upd", argv[5]);
  fp = fopen(fname,"w");

  fprintf(fp,"%d\n", n_ops);
  
  while(n_ins+n_del+n_que)
  {
    int rnd = int(drand48()*(n_ins+n_del+n_que-1));
    if(rnd < n_ins)
    {
      if(del_bridges)
      {
        edge e = random_ins();
	fprintf(fp,"0 %d %d\n", index(source(e)), index(target(e)));
        n_ins--;
      }
      else // no deleted bridges left
      {
        if(n_del) // try a deletion
        {
          edge e = random_del();
	  fprintf(fp,"1 %d %d\n", index(source(e)), index(target(e)));
	  n_del--;
        }
        else // impossible choice of insertions and deletions
        {
          cout << "error error error error\n";
          cerr << "Sorry, Dave. I'm afraid I can't do that.\n";
        }
      }
    }
    else  // rnd >= n_ins
    {
      if(rnd < n_ins + n_del)
      {
        if(bridges)
        {
          edge e = random_del();
	  fprintf(fp,"1 %d %d\n", index(source(e)), index(target(e)));
          n_del--;
        }
        else
        {
          if(n_ins)
          {
            edge e = random_ins();
	    fprintf(fp,"0 %d %d\n", index(source(e)), index(target(e)));
            n_ins--;
          }
          else // impossible choice of insertions and deletions
          {
            cout << "error error error error\n";
            cerr << "Sorry, Dave. I'm afraid I can't do that.\n";
          }
        }
      }
      else // rnd >= n_ins + n_del
      {
	int u, v;
	u = int(drand48()*num_cities)*num_nodes_per_city + 
	  int(drand48()*num_nodes_per_city);
	v = int(drand48()*num_cities)*num_nodes_per_city + 
	  int(drand48()*num_nodes_per_city);
	fprintf(fp,"2 %d %d\n", u, v);
        n_que--;
      }
    }
  }

  // clean up
  delete dummy;
}
  






