#!/usr/local/ts/bin/perl -w
# Replace with your perl path
#
#// ---------------------------------------------------- //
#// gen_line_bs.pl: Generate worst-case input for HDT,   //
#//            consisting of a line (path), on which we  //
#//	       do deletions in "binary search" order.	 //
#//                                                      //
#// usage: gen_line_bs.pl k name                  	 //
#//             k=log(n), n=# of nodes in line		 //
#//             name = name of output files (.gr,.upd)	 //
#// ---------------------------------------------------- //

if( @ARGV == 2) { $k = $ARGV[0]; $name = $ARGV[1]; }
else { 
    print "usage: k name\n";
    print "\tk=log(n), n=# of nodes in line\n";
    print "\tname = name of output files (.gr,.upd)\n";
    exit(-1);
}

open( GR, ">$name.gr" ) || die "can't open $name.gr\n";
open( UPD, ">$name.upd" ) || die "can't open $name.upd\n";

if( $k > 0 ) {
    $n=2**$k;
} else {
    die "k must be greater than 0\n";
}

# print out nodes
print GR "LEDA.GRAPH\nvoid\nvoid\n$n\n";
for($i = 0; $i < $n; $i++) {
    print GR "|\{\}|\n";
}

#print out edges    (nodes numbered 1 to n)
$m=$n-1;
print GR "$m\n";
for($i=1; $i<$n; $i++) {
    print GR "$i ".eval($i+1)." 0 |\{\}|\n";
}
close(GR);

# generate updates
$num_upd = $n-1;
print UPD "$num_upd\n";

$step_size = $n; 
$left_most_node = $step_size/2; 
while( $left_most_node >= 1 ) {
    for( $u=$left_most_node; $u<=$n; $u+=$step_size ) {
# in .upd file, nodes are -1 from .gr file
	$v= $u-1;
	$w= $v+1;   #(v,w) is edge from u to u+1
	print UPD "1 $v $w\n"; # del 
    }
    $step_size = $step_size/2; 
    $left_most_node = $step_size/2; 
} 
close(UPD);

