#ifndef _WEIGHTS_H
#define _WEIGHTS_H

// $Id: dc_hdt.c,v 1.2 1998/03/21 19:10:26 rajiyer Exp rajiyer $

// ------------------------------------------------------------- //
// weights.h:  this file declares the class of weights used by  //
//		the HDT algorithm and defines operators for use  //
//		with hdt_rnbw_trees.				 //
//								 //
// Version 1.0 (July, 2000)                                      //
//								 //
// Implementation by Raj D. Iyer and H. S. Rahul		 //
// There is ABSOLUTELY NO WARRANTY.                              //
// ------------------------------------------------------------- //


class Weight
{
  private:
  // weights are maintained in a single long integer to save space and
  // time.
    unsigned long int data; /* 32 bits, assumed subtree sizes are < 2^30 */
    static const unsigned short int TREE_BIT_SHIFT = 0;
    static const unsigned long int TREE_BIT_MASK = (1 << TREE_BIT_SHIFT);
    static const unsigned short int NON_TREE_BIT_SHIFT = 1;
    static const unsigned long int NON_TREE_BIT_MASK = 
                                                   (1 << NON_TREE_BIT_SHIFT); 
    static const unsigned short int SIZE_SHIFT = 2;
  
public:
    inline int get_size() 
    { 
        return int(data>>SIZE_SHIFT); 
    }

    inline bool get_tree_bit() 
    { 
        return bool((data & TREE_BIT_MASK)>>TREE_BIT_SHIFT); 
    }

    inline bool get_non_tree_bit() 
    {
        return bool((data & NON_TREE_BIT_MASK)>>NON_TREE_BIT_SHIFT); 
    }

    inline void set_size(int val) 
    { 
        data = ((val << SIZE_SHIFT) | 
		(data & TREE_BIT_MASK) | 
		(data & NON_TREE_BIT_MASK));  
    }

    inline void set_tree_bit(bool val) 
    { 
        data = ((data & (~(0xffff & TREE_BIT_MASK))) |
		((val & 1) << TREE_BIT_SHIFT)); 
    }

    inline void set_non_tree_bit(bool val) 
    { 
        data = ((data & (~(0xffff & NON_TREE_BIT_MASK))) | 
		((val & 1) << NON_TREE_BIT_SHIFT)); 
    }

    Weight(int s=1, bool v1 = false, bool v2 = false) 
    {
        data = ((s << SIZE_SHIFT) |
		((v1 & 1) << TREE_BIT_SHIFT) |
		((v2 & 1) << NON_TREE_BIT_SHIFT));
    }

    friend inline Weight operator +(Weight w1, Weight w2)
    {
        Weight w;
	w.data = ((((w1.data >> SIZE_SHIFT) + (w2.data >> SIZE_SHIFT)) 
		   << SIZE_SHIFT) | 
		  (w1.data & TREE_BIT_MASK) |
		  (w2.data & TREE_BIT_MASK) |
		  (w1.data & NON_TREE_BIT_MASK) | 
		  (w2.data & NON_TREE_BIT_MASK));
	return w;
    }

    inline Weight& operator +=(Weight w)
    {
        data = ((((data >> SIZE_SHIFT)+(w.data >> SIZE_SHIFT)) << SIZE_SHIFT) |
		(data & TREE_BIT_MASK) | 
		(w.data & TREE_BIT_MASK) |
		(data & NON_TREE_BIT_MASK) | 
		(w.data & NON_TREE_BIT_MASK));
	return *this;
    }

    inline int operator ==(Weight w)
    {
        return ((data >> SIZE_SHIFT) == (w.data >> SIZE_SHIFT));
    }

    inline int operator <(Weight w)
    {
        return ((data >> SIZE_SHIFT) < (w.data >> SIZE_SHIFT));
    }

    inline int operator <=(Weight w)
    {
	return ((data >> SIZE_SHIFT) <= (w.data >> SIZE_SHIFT));
    }

    inline int operator >(Weight w)
    {
	return ((data >> SIZE_SHIFT) > (w.data >> SIZE_SHIFT));
    }

    inline int operator >=(Weight w)
    {
	return ((data >> SIZE_SHIFT) >= (w.data >> SIZE_SHIFT));
    }

    inline int operator ==(unsigned int n)
    {
	return ((data >> SIZE_SHIFT) == n);
    }

    inline int operator <(unsigned int n)
    {
	return ((data >> SIZE_SHIFT) < n);
    }

    inline int operator <=(unsigned int n)
    {
	return ((data >> SIZE_SHIFT) <= n);
    }

    inline int operator >(unsigned int n)
    {
	return ((data >> SIZE_SHIFT) > n);
    }

    inline int operator >=(unsigned int n)
    {
	return ((data >> SIZE_SHIFT) >= n);
    }

    friend ostream& operator<<(ostream&s, Weight w)
    {
	return s << "(" << (w.data >> SIZE_SHIFT) << "," 
	         << int(w.data & TREE_BIT_MASK) << "," 
		 << int(w.data & NON_TREE_BIT_MASK)<< ")";
    }
};

#endif  /* _WEIGHTS_H  */
